/*  asimodem.c
*
* The Greenleaf Comm Library
*
* Copyright (C) 1985-1990 Greenleaf Software Inc.  All Rights Reserved.
*
*  All Hayes compatible modem control functions are in here.  They are
*  designed to work with modems like the Hayes Smartmodem 1200.
*
*  MODIFICATIONS
*  ""		20-FEB-1987  08:58:17.18
*       Changed calls to asiignor() to asicheck()
*  ""    26-MAY-1988  09:49:03.73  Version 2.20  SAR #91
*    modified hmdialmode(), documentation said 0 == touchtone, 1 == pulse,
*    it actually was working just the opposite.
*
*  Mark Nelson       14-SEP-1989  09:49:03.73  Version 3.00
*
*  Mark Nelson       29-JAN-1989  09:49:03.73  Version 3.00
*
*  added the code to wait after a hayes modem command.  this consists of
*  modifications to hmstr(), and the addition of the two routines called
*  hmfixeddelay() and hmwaitforok().  Also changed the hmdialmode() routine
*  so that it works on an array instead of a global variable.
*  This allows you to support Tone or Pulse dialing on a port by port
*  basis, instead of having a global flag that sets every port.
*
*  Mark Nelson       29-JAN-1989  09:49:03.73  Version 3.01
*
*  Most of the HM... functions were returning the value they got back
*  from HMSendString.  This did not jibe with what the docs said, so
*  they now return ASSUCCESS if HMSendString works, otherwise they return
*  an error code.
*
*/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "gf.h"
#include "asiports.h"
#include "ibmkeys.h"

/*  R E S E T   M O D E M  -  ATZ
 *
 */
int GF_CONV HMReset( int port )
{
    int error_code;

    error_code = HMSendString( port, "ATZ" );
    if (error_code >= ASSUCCESS )
        return( ASSUCCESS );
    else
        return( error_code );
}


/*  C O N V E R T   I N T E G E R   T O   A S C I I,  A N D   C O N C A T.
 *
 */
void GF_CONV icat( char *s, int i )
{
    char buf[ 12 ];

    sprintf( buf, "%d", i );
    strcat( s, buf );
}


/*  S E T   M O D E M   R E G I S T E R    ATSn=v
 *
 */
int GF_CONV HMSetRegister( int port, int num, int val )
{
    char str[ 12 ];
    int error_code;

    strcpy( str, "ATS" );
    icat( str, num );
    strcat( str, "=" );
    icat( str, val );
    error_code = HMSendString( port, str );
    if (error_code >= ASSUCCESS )
        return( ASSUCCESS );
    else
        return( error_code );
}


/*  A U T O  -  A N S W E R   R I N G   C O U N T   ATS0=n
 *
 */
int GF_CONV HMSetAutoAnswerRingCount( int port, int num )
{
    return( HMSetRegister( port, 0, num ) );
}


/*  G E T   R I N G   C O U N T    -   ATS1?
 *
 */
int GF_CONV HMGetIncomingRingCount( int port )
{
    return( HMGetRegister( port, 1 ) );
}


/*  S E T   E S C A P E   C O D E -   ATS2=V
 *
 */
int GF_CONV HMSetEscapeCode( int port, char val )
{
    return( HMSetRegister( port, 2, val ) );
}


/*  S E T  ASCII  V A L U E   O F   E N D - O F - L I N E   C H A R  -  ATS3=v
 *
 */
int GF_CONV HMSetEndOfLineCharacter( int port, char val )
{
    return( HMSetRegister( port, 3, val ) );
}


/*  S E T   ASCII   V A L U E   O F   L I N E F E E D  -  ATS4=v
 *
 */
int GF_CONV HMSetLineFeedCharacter( int port, char val )
{
    return( HMSetRegister( port, 4, val ) );
}


/*  S E T   ASCII   V A L U E   O F   B A C K S P A C E   -  ATS5=v
 *
 */
int GF_CONV HMSetBackspaceCharacter( int port, char val )
{
    return( HMSetRegister( port, 5, val ) );
}


/*  S E T   W A I T   F O R   D I A L   T O N E   T I M E   -  ATS6=v
 *
 */
int GF_CONV HMSetWaitForDialToneTime( int port, int val )
{
    return( HMSetRegister( port, 6, val ) );
}


/*   S E T   W A I T   F O R   C A R R I E R   T I M E   -  ATS7=v
 *
 */
int GF_CONV HMSetWaitTimeForCarrier( int port, int val )
{
    return( HMSetRegister( port, 7, val ) );
}


/*   S E T   P A U S E   T I M E   F O R   C O M M A   -   ATS8=v
*
*/
int GF_CONV HMSetPauseTimeForComma(int port, int val )
{
    return( HMSetRegister( port, 8, val ) );
}


/*   S E T   C A R R I E R   D E T E C T   R E S P O N S E   -   ATS9=v
 *
 */
int GF_CONV HMSetCDResponseTime( int port, int val )
{
    return( HMSetRegister( port, 9, val ) );
}


/*  S E T   C A R R I E R   D I S C.   T I M E   -   ATS10=v
 *
 */
int GF_CONV HMSetCarrierDisconnectTime( int port, int val )
{
    return( HMSetRegister( port, 10, val ) );
}


/*  S E T   D I A L   S P E E D   ( T O U C H T O N E )  -  ATS11=v
 *
 */
int GF_CONV HMSetTouchToneDuration( int port, int val )
{
    return (HMSetRegister( port, 11, val ) );
}


/*  S E T   E S C A P E   G U A R D   T I M E   -   ATS12=v
 *
 */
int GF_CONV HMSetEscapeCodeGuardTime( int port, int val )
{
    return( HMSetRegister( port, 12, val ) );
}

/*   R E A D   U A R T   S T A T U S   R E G  -- ATS13?
 *
 */
int GF_CONV HMGetUARTStatus( int port )
{
    return( HMGetRegister( port, 13 ) );
}

/*   R E A D   M O D E M   O P T I O N   R E G   -- ATS14?
 *
 */
int GF_CONV HMGetOptionRegister( int port )
{
    return( HMGetRegister( port, 14 ) );
}

/*   R E A D   M O D E M   F L A G   R E G   -   ATS15?
 *
 */
int GF_CONV HMGetFlagRegister( int port )
{
    return( HMGetRegister( port, 15 ) );
}


/*  G O   T O   O N L I N E   S T A T E  -  ATO
 *
 */
int GF_CONV HMGoOnline( int port )
{
    int error_code;

    error_code = HMSendStringNoWait( port, "ATO", '\r' ) ;
    if (error_code >= ASSUCCESS )
        return( ASSUCCESS );
    else
        return( error_code );
}

/*  S E T   D I A L   M O D E   ( PULSE / TOUCH )
 *
 *
 *  1==PULSE or 0==DIAL  Sends the command to the modem causing all future
 *                       dialing to adopt the new mode.
 */

static char _dialmode[MAX_PORT];

int GF_CONV HMSetDialingMethod( int port, int ctrl )
{
    if ( port >=0 && port < MAX_PORT && ctrl >= TOUCH_TONE && ctrl <= PULSE) {
        _dialmode[port] = ( char ) ctrl;
        return( ASSUCCESS );
    }
    else
        return( ASINVPAR );
}

/*  D I A L   P H O N E   N U M B E R  -  ATDmXXXXXXX
 *
 *  Dial Number from ASCII string.  String must be terminated.
 *  May contain seperators.  "," causes delay.
 *  Ignores any non-numeric characters other than comma.
 */
int GF_CONV HMDial( int port, char *s)
{
    char str[ 50 ];
    int error_code;


    if ( _dialmode[ port ] == PULSE )
        strcpy( str, "ATDP" );
    else
        strcpy( str, "ATDT" );
    strcat( str, s );
    error_code = HMSendStringNoWait( port, str, '\r' );
    if (error_code >= ASSUCCESS )
        return( ASSUCCESS );
    else
        return( error_code );
}


/*  R E P E A T   L A S T   A/
 *
 */
int GF_CONV HMRepeatLastCommand( int port )
{
    int error_code;

    error_code = HMSendStringNoWait( port, "A/", -1 );
    if (error_code >= ASSUCCESS )
        return( ASSUCCESS );
    else
        return( error_code );
}

/*  D I A L   O R I G I N A T E   O N L Y   R E M O T E   M O D E M
 *
 */
int GF_CONV HMDialInAnswerMode( int port, char *s )
{
    char str[ 50 ];
    int error_code;


    if ( _dialmode[ port ] == PULSE )
        strcpy( str, "ATDP" );
    else
        strcpy( str, "ATDT" );
    strcat( str, s );
    strcat( str, "R" );
    error_code = HMSendStringNoWait( port, str, '\r' );
    if (error_code >= ASSUCCESS )
        return( ASSUCCESS );
    else
        return( error_code );
}

int GF_CONV HMDialAndReturnToCommandMode( int port, char *s )
{
    char str[ 50 ];
    int error_code;


    if ( _dialmode[ port ] == PULSE )
        strcpy( str, "ATDP" );
    else
        strcpy( str, "ATDT" );
    strcat( str, s );
    strcat( str, ";" );
    error_code = HMSendStringNoWait( port, str, '\r' );
    if (error_code >= ASSUCCESS )
        return( ASSUCCESS );
    else
        return( error_code );
}


/*  A N S W E R   P H O N E   -  ATA
 *
 */
int GF_CONV HMAnswer( int port )
{
    int error_code;

    error_code = HMSendStringNoWait( port, "ATA", '\r' );
    if (error_code >= ASSUCCESS )
        return( ASSUCCESS );
    else
        return( error_code );
}

/*  Send string followed by ASCII digit (from integer arg3)
 *
 *
 */
int GF_CONV _hm3( int port, char *s, int ctrl )
{
    char buf[ 40 ];
    int error_code;


    strcpy( buf, s );
    icat( buf, ctrl );
    error_code = HMSendString( port, buf );
    if (error_code >= ASSUCCESS )
        return( ASSUCCESS );
    else
        return( error_code );
}


/*  T U R N   C A R R I E R   ON / OFF  -  ATCn
 *
 */
int GF_CONV HMSetCarrier( int port, int ctrl )
{
    if ( ctrl < 0 || ctrl > 1 )
        return ( ASINVPAR );
    else
        return( _hm3( port, "ATC", ctrl ) );
}

/*  E C H O   ON / OFF  -  ATEn
 *
 */
int GF_CONV HMSetEchoMode( int port, int ctrl )
{
    if ( ctrl < 0 || ctrl > 1 )
        return( ASINVPAR );
    else
        return( _hm3( port, "ATE", ctrl ) );
}

/*  F U L L  /  H A L F   D U P L E X   -   ATFn
 *
 */
int GF_CONV HMSetFullDuplexMode( int port, int ctrl )
{
    if ( ctrl < 0 || ctrl > 1 )
        return ( ASINVPAR );
    else
        return( _hm3( port, "ATF", ctrl ) );
}

/*   O N   /   O F F   H O O K, OR SPECIAL OFFHOOK FOR HAM RADIO APPL.
 *
 *  0=OnHook 1=OffHook 2=Special OffHook
 */
int GF_CONV HMSetHookSwitch(int port, int ctrl )
{
    if ( ctrl < 0 || ctrl > 2 )
        return( ASINVPAR );
    else
        return( _hm3( port, "ATH", ctrl ) );
}

/*
 *  Control Monitor Speaker:  0=Off 1=On until CD, 2=Always On - Mn
 */
int GF_CONV HMSetSpeaker( int port, int ctrl )
{
    if ( ctrl < 0 || ctrl > 2 )
        return ( ASINVPAR );
    else
        return( _hm3( port, "ATM", ctrl ) );
}

/*
 *  Quiet Mode (Result Code Generation) 1=Quiet 0=Send Results - Qn
 */
int GF_CONV HMReturnNoResultCodes( int port, int ctrl )
{
    if ( ctrl < 0 || ctrl > 1 )
        return ( ASINVPAR );
    else
        return( _hm3( port, "ATQ", ctrl ) );
}

/*  Extended Result Codes -Xn
 */
int GF_CONV HMSelectExtendedResultCodes( int port, int ctrl )
{
    if ( ctrl < 0 || ctrl > 1 )
        return ( ASINVPAR );
    else
        return( _hm3( port, "ATX", ctrl ) );
}

/*
 *  Verbose Mode (Result Codes) 0=Digits 1=Words - Vn
 */

int GF_CONV HMSetVerboseMode( int port, int ctrl )
{
    if ( ctrl < 0 || ctrl > 1 )
        return ( ASINVPAR );
    else
        return( _hm3( port, "ATV", ctrl ) );
}

/*
 * HMWaitForOK( int ticks, char *match_string )
 *
 * This routine is used to set up the semi-intelligent Hayems modem delay.
 * After every command, the HMSendString() routine will sit in a loop for the number
 * of clock ticks specified here.  It will read input characters until if
 * finds one that looks like the match string.  As it reads in the chars,
 * they are printed out using the character printer routine.  If the routine
 * is NULL, they are not printed.  If the match string is null, the default
 * string of "OK" is used.
 */

static int hm_delay_value=0;
static char *hm_match_string="OK";
static void (GF_CDECL *hm_character_printer)(char c)=NULL;
static unsigned int hm_abort_key=NOKEY;

void GF_CONV HMWaitForOK( int ticks, char *match_string )
{
    if ( match_string != NULL )
        hm_match_string = match_string;
    hm_delay_value = -ticks;
}

/*
 * void HMSetUpEchoRoutine( void ( *character_printer )( char c ) )
 *
 * When characters are being read in in HMInputLine, the character printer
 * routine can be used to echo them out.  This is how the character printer
 * routine is set up.
 */
void GF_CONV HMSetUpEchoRoutine( void ( GF_CDECL *character_printer )( char c ) )
{
    hm_character_printer = character_printer;
}

/*
 * void HMSetUpAbortKey( unsigned int key )
 *
 * When characters are being read in in HMInputLine, the user can hit an abort
 * key and cause an immediate return, with a return value of ASUSERABORT.
 * This is the routine where the abort key can be set up.
 */
void GF_CONV HMSetUpAbortKey( unsigned int key )
{
    hm_abort_key = key;
}

/*
 *  HMFixedDelay( int ticks ) - This routine is called once to establish a
 *  fixed delay invoked after every call to HMSendString().  The user can set
 *  this up once with a call, and then does not have to bother with
 *  a timer instruction after every hm*() call.  This is one of the two
 *  methods used to make sure that the modem buffer does not
 *  overflow because commands are sent too rapidly.
 */

void GF_CONV HMFixedDelay( int ticks )
{
    hm_delay_value = ticks;
}

/*  HMSendString()  - a primitive to send strings to the modem.  If interrupts
 *  are running uses asiputs(), if not, uses asputs().  Also, we make sure
 *  that we send the string regardless of the state of CTS and DSR, which
 *  some modems do not assert until in the online state.  This is done by
 *  checking to see if each is currently checked, and if so, calling
 *  asicheck() to ignore it.  If checking for DSR and/or CTS on entry,
 *  we remember to call asicheck() again while leaving to restore checking.
 *  This routine now has two optional delay methods it can use after sending
 *  a string to the modem.  If the global delay parameter is set to 0, no
 *  delay is performed.  This is compatible with previous versions of the
 *  library, and essentially leaves the responsibility for delay up to
 *  the programmer.  If the delay parameter is positive, a fixed delay is
 *  executed after the command is sent.  If the delay is negative, it is
 *  considered to be a timeout value, and while the timeout is not up
 *  the program waits for an "OK" string, or a specified user defined
 *  string.
 */
int GF_CONV HMSendString( int port, char *string )
{
    int current_time;
    int retvalue;
    char buffer[ 40 ];

    retvalue = HMSendStringNoWait( port, string, '\r' );
    if ( retvalue < ASSUCCESS )
        return( retvalue );
/*
 * If hm_delay_value is 0, we are going to let the calling program be
 * responsible for modem pacing.
 */
    if ( hm_delay_value == 0 )
        return( retvalue );
/*
 * If hm_delay_value is greater than 0, then modem pacings consists of
 * a fixed number of 55 ms ticks to be killed after each modem command.
 */
    if ( hm_delay_value > 0 ) {
        timer( hm_delay_value );
        return( retvalue );
    }
/*
 * If we fall through to here, it means the modem pacing is going to be
 * the pseudo-intelligent method of waiting for an appropriate answer
 * string.  Note that I still execute a short delay after getting an "OK".
 * Most of the modems we talk to are somewhat brain-damaged, and need
 * lots of extra time to think things over after saying OK.
 */
    current_time = -hm_delay_value;
    if ( hm_match_string == NULL )
        return( retvalue );
    for ( ; ; ) {
        current_time = HMInputLine( port, current_time, buffer, 40 );
        if ( current_time < 0 )
            return( current_time );
        if ( current_time == 0 )
            return( ASNOHAYESOK );
        if ( strcmp( hm_match_string, buffer ) == 0) {
            timer( 9 );
            return( retvalue );
        }
    }
}

/*
 * This is the simple routine that just performs string output.  If you
 * send a command that doesn't generate an "OK" message, like a dial
 * or answer command, this routine can be called to prevent pacing.
 */
int GF_CONV HMSendStringNoWait( int port, char *string, int termination_sequence )
{
    int wascts;
    int retvalue;
    int wasdsr;
    int wascd;

    wascts = CTSLowHoldsTXInterrupts( port );
    if ( wascts < 0 )
        return(wascts);
    wasdsr = DSRLowDiscardsRXData( port );
    if ( wasdsr < 0 )
        return ( wasdsr );
    wascd = CDLowDiscardsRXData( port );
    if ( wascd < 0 )
        return ( wascd );
    asicheck( port, CTS_LOW_STOPS_TX_INTERRUPTS, OFF );
    asicheck( port, CD_LOW_DISCARDS_RX_DATA, OFF );
    asicheck( port, DSR_LOW_DISCARDS_RX_DATA, OFF );
    if( istxintrunning( port ) == TRUE )
        retvalue = ( int ) asiputs( port, string, termination_sequence );
    else
        retvalue = ( int ) asputs( port, string, termination_sequence );
    while ( !istxempty( port ) )
        ;
    if ( wascts )
        asicheck( port, CTS_LOW_STOPS_TX_INTERRUPTS, ON );
    if ( wasdsr )
        asicheck( port, DSR_LOW_DISCARDS_RX_DATA, ON );
    if ( wascd )
        asicheck( port, CD_LOW_DISCARDS_RX_DATA, ON );
    return( retvalue );
}

/*  R E A D   M O D E M   R E G I S T E R   ATSn?
 *
 */
int GF_CONV HMGetRegister( int port, int num )
{
    char buffer[ 40 ];
    int status;
    unsigned char return_value;
    int ticks_left;
    int i;

    strcpy( buffer, "ATS" );
    icat( buffer, num );
    strcat( buffer, "?" );
    status = HMSendStringNoWait( port, buffer, '\r' );
    if ( status < 0)
        return( status );
    ticks_left=24;
    for ( ; ; ) {
        ticks_left = HMInputLine( port, ticks_left, buffer, 40 );
        if ( ticks_left < 0 )
            return(ticks_left);
        if (ticks_left == 0 )
            return( ASNOHAYESRESPONSE );
        return_value = 0;
        for ( i = 0 ; i < (int)strlen( buffer ) ; i++ ) {
            if ( !isdigit( buffer[ i ] ) )
               break;
            return_value *= 10;
            return_value += buffer[ i ] - '0';
        }
        if (i > 0 && i == (int)strlen( buffer ))
            break;
    }
    status = HMSendString( port, "" );
    if ( status < 0 )
        return( status );
    return( return_value );
}

int GF_CONV HMInputLine( int port, int ticks, char *buffer, int length )
{
    int wasdsr;
    int wascd;
    int i;
    int c;
    unsigned int key;

    wasdsr = DSRLowDiscardsRXData( port );
    if ( wasdsr < 0 )
        return ( wasdsr );
    wascd = CDLowDiscardsRXData( port );
    if ( wascd < 0 )
        return ( wascd );

    asicheck( port, CD_LOW_DISCARDS_RX_DATA, OFF );
    asicheck( port, DSR_LOW_DISCARDS_RX_DATA, OFF );

    i = 0;
    length--;
    if ( length <= 0 )
        return( ASINVPAR );
    for ( ; ; ) {
       if ( hm_abort_key != NOKEY ) {
           while ( gfkbhit() ) {
               key = getkey();
               if ( key == hm_abort_key )
                   return( ASUSERABORT );
            }
        }
        c = asigetc( port );
        if ( c >= 0 ) {
            if ( hm_character_printer != NULL )
                hm_character_printer( ( char ) c );
            if ( c == '\r' )
                break;
            if ( c == '\n' )
                continue;
            buffer[ i++ ] = ( char ) c;
            length--;
            if ( length <= 0 )
                break;
        } else if ( c == ASBUFREMPTY ) {
            timer( 1 );
            ticks--;
            if ( ticks <= 0 )
                break;
        } else {
            ticks = c;
            break;
        }
    }
    buffer[ i ] = '\0';
    if ( wasdsr )
        asicheck( port, DSR_LOW_DISCARDS_RX_DATA, ON );
    if ( wascd )
        asicheck( port, CD_LOW_DISCARDS_RX_DATA, ON );
    return( ticks );
}
