/*  asishare.c
 *
 * The Greenleaf Comm Library
 *
 * Copyright (C) 1985-1990 Greenleaf Software Inc.  All Rights Reserved.
 *
 *  int asishare( mask )
 *  unsigned mask;      -- Mask for bits to mask as_sbmask
 *
 *  This function customizes the interrupt routines for use with a particular
 *  Category B board by arranging that the status port data will be masked
 *  before being compared with the "sh_bmask" argument to asisetv().
 *
 *  Because some boards (e.g. QuadBoard-AT) have bits that are high all the
 *  time, it is necessary that the interrupt not see them.
 *
 *  NOTE:  All ports must use the same mask, this does not mean that all
 *      ports must reside on shared interrupt boards, only that all of
 *      the boards must be of the same type.
 *
 *  RETURNS:
 *              ASSUCCESS always.
 *
 *  USAGE:
 *              Call AFTER asifirst() for at least one port.
 *              Call only once (other calls ignored).
 *
 * MODIFICATIONS
 *  David Nienhiser   04-FEB-1987  16:42:33.14
 *      Modified to save mask value in global variable _asmask (in asifirs.c).
 *      If the data structures have not been set up this function returns
 *      ASSUCCESS.  It will then be up to asifirst() to set the value in
 *      the data structure when it is built.  If the data structure has been
 *      setup asishare will fill all values in as before.
 *
 */

#include <stdio.h>
#include "gf.h"
#include "asiports.h"

int GF_CONV asishare( unsigned mask )
{
        int i;

        _asmask=mask;
        if(as_chnl)
                for (i=0; i<MAX_PORT; i++)
                        as_chnl->tblport[i].as_mask = mask;
        return (ASSUCCESS);
}
