/* gets.c
*
* The Greenleaf Comm Library
*
* Copyright (C) 1985-1990 Greenleaf Software Inc.  All Rights Reserved.
*
*  unsigned asigets(port,string,length,opt)
*  int port;            - Port 0..MAX_PORT-1
*  char *string;        - String to transfer
*  unsigned length;     - Maximum number of characters to transfer.
*  int opt;             - Terminating character to watch for.
*
* DESCRIPTION
*  a string is transferred one character at a time from the receive
*  queue to the specified user buffer.  The terminating null is NOT
*  appended to the user buffer.
*
* SIDE EFFECTS
*  none
*
* RETURNS
*  Always returns the number of characters transferred to the user
*  buffer NOT INCLUDING the terminating null.  For example if only
*  one character was received from the receive queue a null would
*  be added to the second position of the buffer and a value of 1
*  would be returned.  The global variable _aserror can be examined to
*  determine if an error was detected.  _aserror == 0 if no errors were
*  detected.
*
*       _aserror =
*
*       Value           Meaning
*     -------          --------
*       ASSUCCESS       port initialized (no error)
*       ASINVPORT       Requested port is out of range
*       ASNOTSETUP      Requested port not setup with asifirst()
*       ASINVPAR        Option parameter invalid
*       ASBUFREMPTY     Buffer is empty
*
*       Also see possible error returns for asigetc().
*
* MODIFICATIONS
*
*  11-01-85     David Nienhiser
*               Modified for release 2.0
*  21-JUN-88    Kamy Rahimi
*               According to the manual, now the terminator is not transferred
*               to the destination string.
*/
#include <stdio.h>
#include "gf.h"
#include "asiports.h"

unsigned int GF_CONV asigets( int port,
                      char *string,
                      unsigned int length,
                      int opt)
{
    return( asigets_timed( port, string, length, opt, 0 ) );
}

